#include <stdio.h>
#include <conio.h>
#include <io.h>
#include <dir.h>
#include <alloc.h>
#include <dspi.h>

#ifdef __DPMI32__
	#define  CONFIG   "MIDI32.CFG"
	#define  DRV      "..\\32BIT"
#elif defined(__DPMI16__)
	#define  CONFIG   "MIDI16.CFG"
	#define  DRV      "..\\16BIT"
#else
	#define  CONFIG   "MIDI.CFG"
	#define  DRV      "..\\REAL"
#endif

LPBYTE GetConfigString (void);

void main(int argc, char** argv) {
	if (argc != 2) {
		printf ("Usage: MIDIPLAY filename\n");
		return;
	}

	LPBYTE lpString = GetConfigString();
	if ( lpString == NULL ) {
		printf ("ERROR: Unable to load file %s\n", CONFIG);
		return;
	}

	chdir(DRV);
	if ( midiLoadDriver((LPSTR)lpString) != MIDI_ERROR_NONE ) {
		printf ("ERROR: Unable to load driver %s\n", lpString);
		chdir ("..");
		return;
	}
	chdir ("..\\BIN");
	midiSetDriverSpecific ( lpString + 13 );
	if ( midiDriverEntry() != MIDI_ERROR_NONE ) {
		printf ("ERROR: Cannot initialize driver\n");
		return;
	}

	MIDICAPS Caps;
	midiGetDeviceCaps(&Caps);
	clrscr();
	printf ("Version: %u.%.2u\n", (Caps.wVersion) >> 8, (Caps.wVersion) & 0xFF );
	printf ("Name: %s\n", Caps.lpszDriverName);
	printf ("Manufacturer: %s\n", Caps.lpszManufacturer);
	printf ("Description: %s\n", Caps.lpszDescription);
	printf ("Channels: %d\n", Caps.wChannels);
	printf ("Polyphony: %d\n", Caps.wMaximumPolyphony);
	printf ("Technology: %d\n", Caps.wTechnology);
	SMF_0 MidiFile;
	HPBYTE lpFile = midiLoadSMF ( argv[1], &MidiFile );
	if (lpFile == NULL) {
		printf ("ERROR: Cannot find file %s\n", argv[1] );
		return;
	}
	printf ("\nFile header: %.4s\n", MidiFile.MThd);
	printf ("Header length: %lu\n", MidiFile.dwHeaderLength);
	printf ("Type: %u\n", MidiFile.wType);
	printf ("Tracks: %u\n", MidiFile.wTracks);
	printf ("Ticks per quarter: %u\n", MidiFile.wTicksPerQuarter);
	printf ("Track ID: %.4s\n", MidiFile.MTrk);
	printf ("Track length: %lu\n", MidiFile.dwTrackLength);
	midiPlaySMF (lpFile, &MidiFile);
	do {
		if ( kbhit() ) {
			getch();
			midiFadeOut(4);
		}
	} while ( midiGetStatus() != MIDI_STOPPED );
	midiCloseDriver();
}

LPBYTE GetConfigString (void) {
	FILE *fp;
	fp = fopen (CONFIG, "rb");
	if (fp == NULL) {
		return NULL;
	}
	DWORD dwLen = filelength(fileno(fp));
	LPBYTE file = (LPBYTE)farmalloc(dwLen + 1);
	fread (file, 1, (WORD)dwLen, fp);
	fclose (fp);
	return file;
}



